"""Authentication module for future OAuth2 credentials."""


class OAuth2Credentials:
    """Stub for OAuth2 client credentials."""

    def __init__(self):
        raise NotImplementedError("OAuth2Credentials is not implemented yet")
